@@ -51,11 +51,8 @@ __all__ = ['mollview', 'gnomview', 'cartview',
            'projplot', 'projscatter', 'projtext']
 
 import projaxes as PA
-import pylab
 import numpy as np
 import matplotlib
-import matplotlib.colors as colors
-import matplotlib.cbook as cbook
 import pixelfunc
 
 pi = np.pi
@@ -68,13 +65,14 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
              gal_cut=0,
              format='%g',format2='%g',
              cbar=True,cmap=None, notext=False,
-             norm=None,hold=False,margins=None,sub=None):
+             norm=None,hold=False,margins=None,sub=None,
+             return_projected_map=False):
     """Plot an healpix map (given as an array) in Mollweide projection.
     
     Parameters
     ----------
     map : float, array-like or None
-      An array containing the map.
+      An array containing the map, supports masked maps, see the `ma` function.
       If None, will display a blank map, useful for overplotting.
     fig : int or None, optional
       The figure number to use. Default: create a new figure
@@ -132,12 +130,15 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
       giving the margins on left,bottom,right and top
       of the axes. Values are relative to figure (0-1).
       Default: None
+    return_projected_map : bool
+      if True returns the projected map in a 2d numpy array
 
     See Also
     --------
     gnomview, cartview
     """
     # Create the figure
+    import pylab
     if not (hold or sub):
         f=pylab.figure(fig,figsize=(8.5,5.4))
         extent = (0.02,0.05,0.96,0.9)
@@ -177,6 +178,7 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
         if map is None:
             map = np.zeros(12)+np.inf
             cbar=False
+        map = pixelfunc.ma_to_array(map)
         ax=PA.HpxMollweideAxes(f,extent,coord=coord,rot=rot,
                                format=format2,flipconv=flip)
         f.add_axes(ax)
@@ -187,7 +189,7 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
         elif remove_mono:
             map=pixelfunc.remove_monopole(map,gal_cut=gal_cut,nest=nest,
                                           copy=True,verbose=True)
-        ax.projmap(map,nest=nest,xsize=xsize,coord=coord,vmin=min,vmax=max,
+        img = ax.projmap(map,nest=nest,xsize=xsize,coord=coord,vmin=min,vmax=max,
                    cmap=cmap,norm=norm)
         if cbar:
             im = ax.get_images()[0]
@@ -218,7 +220,8 @@ def mollview(map=None,fig=None,rot=None,coord=None,unit='',
         if wasinteractive:
             pylab.ion()
             #pylab.show()
-
+    if return_projected_map:
+        return img
 
 def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
              xsize=200,ysize=None,reso=1.5,degree=False,
@@ -227,13 +230,15 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
              min=None,max=None,flip='astro',
              format='%.3g',cbar=True,
              cmap=None, norm=None,
-             hold=False,sub=None,margins=None,notext=False):
+             hold=False,sub=None,margins=None,notext=False,
+             return_projected_map=False):
     """Plot an healpix map (given as an array) in Gnomonic projection.
 
     Parameters
     ----------
     map : array-like
-      The map to project. If None, use a blank map, useful for
+      The map to project, supports masked maps, see the `ma` function.
+      If None, use a blank map, useful for
       overplotting.
     fig : None or int, optional
       A figure number. Default: None= create a new figure
@@ -290,11 +295,14 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
       Default: None
     notext: bool, optional
       If True: do not add resolution info text. Default=False
+    return_projected_map : bool
+      if True returns the projected map in a 2d numpy array
 
     See Also
     --------
     mollview, cartview
     """
+    import pylab
     if not (hold or sub):
         f=pylab.figure(fig,figsize=(5.8,6.4))
         if not margins:
@@ -337,6 +345,7 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
         if map is None:
             map = np.zeros(12)+np.inf
             cbar=False
+        map = pixelfunc.ma_to_array(map)
         ax=PA.HpxGnomonicAxes(f,extent,coord=coord,rot=rot,
                               format=format,flipconv=flip)
         f.add_axes(ax)
@@ -344,7 +353,7 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
             map=pixelfunc.remove_dipole(map,gal_cut=gal_cut,nest=nest,copy=True)
         elif remove_mono:
             map=pixelfunc.remove_monopole(map,gal_cut=gal_cut,nest=nest,copy=True)
-        ax.projmap(map,nest=nest,coord=coord,vmin=min,vmax=max,
+        img = ax.projmap(map,nest=nest,coord=coord,vmin=min,vmax=max,
                    xsize=xsize,ysize=ysize,reso=reso,cmap=cmap,norm=norm)
         if cbar:
             im = ax.get_images()[0]
@@ -384,7 +393,8 @@ def gnomview(map=None,fig=None,rot=None,coord=None,unit='',
         if wasinteractive:
             pylab.ion()
             #pylab.show()
-
+    if return_projected_map:
+        return img
 
 def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
              xsize=800,ysize=None,lonra=None,latra=None,
@@ -393,13 +403,15 @@ def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
              min=None,max=None,flip='astro',
              format='%.3g',cbar=True,
              cmap=None, norm=None,aspect=None,
-             hold=False,sub=None,margins=None,notext=False):
+             hold=False,sub=None,margins=None,notext=False,
+             return_projected_map=False):
     """Plot an healpix map (given as an array) in Cartesian projection.
 
     Parameters
     ----------
     map : float, array-like or None
-      An array containing the map.
+      An array containing the map, 
+      supports masked maps, see the `ma` function.
       If None, will display a blank map, useful for overplotting.
     fig : int or None, optional
       The figure number to use. Default: create a new figure
@@ -459,11 +471,14 @@ def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
       giving the margins on left,bottom,right and top
       of the axes. Values are relative to figure (0-1).
       Default: None
+    return_projected_map : bool
+      if True returns the projected map in a 2d numpy array
 
     See Also
     --------
     mollview, gnomview
     """
+    import pylab
     if not (hold or sub):
         f=pylab.figure(fig,figsize=(8.5,5.4))
         if not margins:
@@ -506,6 +521,7 @@ def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
         if map is None:
             map = np.zeros(12)+np.inf
             cbar=False
+        map = pixelfunc.ma_to_array(map)
         if zat and rot:
             raise ValueError('Only give rot or zat, not both')
         if zat:
@@ -519,7 +535,7 @@ def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
             map=pixelfunc.remove_dipole(map,gal_cut=gal_cut,nest=nest,copy=True)
         elif remove_mono:
             map=pixelfunc.remove_monopole(map,gal_cut=gal_cut,nest=nest,copy=True)
-        ax.projmap(map,nest=nest,coord=coord,vmin=min,vmax=max,
+        img = ax.projmap(map,nest=nest,coord=coord,vmin=min,vmax=max,
                    xsize=xsize,ysize=ysize,lonra=lonra,latra=latra,
                    cmap=cmap,norm=norm,aspect=aspect)
         if cbar:
@@ -550,6 +566,8 @@ def cartview(map=None,fig=None,rot=None,zat=None,coord=None,unit='',
             pylab.ion()
             pylab.draw()
             #pylab.show()
+    if return_projected_map:
+        return img
 
 def graticule(dpar=None,dmer=None,coord=None,local=None,**kwds):
     """Draw a graticule on the current Axes.
@@ -573,6 +591,7 @@ def graticule(dpar=None,dmer=None,coord=None,local=None,**kwds):
     --------
     delgraticules
     """
+    import pylab
     f = pylab.gcf()
     wasinteractive = pylab.isinteractive()
     pylab.ioff()
@@ -590,7 +609,6 @@ def graticule(dpar=None,dmer=None,coord=None,local=None,**kwds):
         pylab.draw()
         if wasinteractive:
             pylab.ion()
-            #pylab.show()
     
 def delgraticules():
     """Delete all graticules previously created on the Axes.
@@ -599,6 +617,7 @@ def delgraticules():
     --------
     graticule
     """
+    import pylab
     f = pylab.gcf()
     wasinteractive = pylab.isinteractive()
     pylab.ioff()
@@ -613,6 +632,7 @@ def delgraticules():
             #pylab.show()
 
 def projplot(*args,**kwds):
+    import pylab
     f = pylab.gcf()
     wasinteractive = pylab.isinteractive()
     pylab.ioff()
@@ -630,6 +650,7 @@ def projplot(*args,**kwds):
 projplot.__doc__ = PA.SphericalProjAxes.projplot.__doc__
     
 def projscatter(*args,**kwds):
+    import pylab
     f = pylab.gcf()
     wasinteractive = pylab.isinteractive()
     pylab.ioff()
@@ -647,6 +668,7 @@ def projscatter(*args,**kwds):
 projscatter.__doc__ = PA.SphericalProjAxes.projscatter.__doc__
 
 def projtext(*args,**kwds):
+    import pylab
     f = pylab.gcf()
     wasinteractive = pylab.isinteractive()
     pylab.ioff()
@@ -662,5 +684,3 @@ def projtext(*args,**kwds):
             #pylab.show()
     return ret
 projtext.__doc__ = PA.SphericalProjAxes.projtext.__doc__
-
-
